@@ -7,14 +7,14 @@ Tests that apply to the general ElementTree API should go into
 test_elementtree
 """
 
-import os.path, unittest, copy, sys, operator
+import os.path, unittest, copy, sys, operator, tempfile, gzip
 
 this_dir = os.path.dirname(__file__)
 if this_dir not in sys.path:
     sys.path.insert(0, this_dir) # needed for Py3
 
 from common_imports import etree, StringIO, BytesIO, HelperTestCase, fileInTestDir
-from common_imports import LargeFileLikeUnicode, doctest, make_doctest
+from common_imports import SillyFileLike, LargeFileLikeUnicode, doctest, make_doctest
 from common_imports import canonicalize, sorted, _str, _bytes
 
 print("")
@@ -154,6 +154,25 @@ class ETreeOnlyTestCase(HelperTestCase):
         self.assertRaises(ValueError, QName, 'na me')
         self.assertRaises(ValueError, QName, 'test', ' name')
 
+    def test_qname_namespace_localname(self):
+        # ET doesn't have namespace/localname properties on QNames
+        QName = self.etree.QName
+        namespace, localname = 'http://myns', 'a'
+        qname = QName(namespace, localname)
+        self.assertEquals(namespace, qname.namespace)
+        self.assertEquals(localname, qname.localname)
+
+    def test_qname_element(self):
+        # ET doesn't have namespace/localname properties on QNames
+        QName = self.etree.QName
+        qname1 = QName('http://myns', 'a')
+        a = self.etree.Element(qname1, nsmap={'p' : 'http://myns'})
+
+        qname2 = QName(a)
+        self.assertEquals(a.tag, qname1.text)
+        self.assertEquals(qname1.text, qname2.text)
+        self.assertEquals(qname1, qname2)
+
     def test_qname_text_resolve(self):
         # ET doesn't resove QNames as text values
         etree = self.etree
@@ -214,6 +233,37 @@ class ETreeOnlyTestCase(HelperTestCase):
         self.assertEquals('ONE',     a.text)
         self.assertEquals('ANOTHER', b.text)
 
+    def test_deepcopy_elementtree_pi(self):
+        XML = self.etree.XML
+        tostring = self.etree.tostring
+        root = XML(_bytes("<?mypi my test ?><test/><!--comment -->"))
+        tree1 = self.etree.ElementTree(root)
+        self.assertEquals(_bytes("<?mypi my test ?><test/><!--comment -->"),
+                          tostring(tree1))
+
+        tree2 = copy.deepcopy(tree1)
+        self.assertEquals(_bytes("<?mypi my test ?><test/><!--comment -->"),
+                          tostring(tree2))
+
+        root2 = copy.deepcopy(tree1.getroot())
+        self.assertEquals(_bytes("<test/>"),
+                          tostring(root2))
+
+    def test_deepcopy_elementtree_dtd(self):
+        XML = self.etree.XML
+        tostring = self.etree.tostring
+        xml = _bytes('<!DOCTYPE test [\n<!ENTITY entity "tasty">\n]>\n<test/>')
+        root = XML(xml)
+        tree1 = self.etree.ElementTree(root)
+        self.assertEquals(xml, tostring(tree1))
+
+        tree2 = copy.deepcopy(tree1)
+        self.assertEquals(xml, tostring(tree2))
+
+        root2 = copy.deepcopy(tree1.getroot())
+        self.assertEquals(_bytes("<test/>"),
+                          tostring(root2))
+
     def test_attribute_set(self):
         # ElementTree accepts arbitrary attribute values
         # lxml.etree allows only strings
@@ -430,6 +480,11 @@ class ETreeOnlyTestCase(HelperTestCase):
         self.assertRaises(
             LookupError, self.etree.XMLParser, encoding="hopefully unknown")
 
+    def test_parser_encoding(self):
+        self.etree.XMLParser(encoding="ascii")
+        self.etree.XMLParser(encoding="utf-8")
+        self.etree.XMLParser(encoding="iso-8859-1")
+
     def test_elementtree_parser_target_type_error(self):
         assertEquals = self.assertEquals
         assertFalse  = self.assertFalse
@@ -667,6 +722,48 @@ class ETreeOnlyTestCase(HelperTestCase):
         root = tree.getroot()
         self.assertEquals(root.text, test_url)
 
+    def test_resolve_bytes_dtd(self):
+        parse = self.etree.parse
+        parser = self.etree.XMLParser(dtd_validation=True)
+        assertEqual = self.assertEqual
+        test_url = _str("__nosuch.dtd")
+
+        class MyResolver(self.etree.Resolver):
+            def resolve(self, url, id, context):
+                assertEqual(url, test_url)
+                return self.resolve_string(
+                    (_str('''<!ENTITY myentity "%s">
+                             <!ELEMENT doc ANY>''') % url).encode('utf-8'),
+                    context)
+
+        parser.resolvers.add(MyResolver())
+
+        xml = _str('<!DOCTYPE doc SYSTEM "%s"><doc>&myentity;</doc>') % test_url
+        tree = parse(StringIO(xml), parser)
+        root = tree.getroot()
+        self.assertEquals(root.text, test_url)
+
+    def test_resolve_filelike_dtd(self):
+        parse = self.etree.parse
+        parser = self.etree.XMLParser(dtd_validation=True)
+        assertEqual = self.assertEqual
+        test_url = _str("__nosuch.dtd")
+
+        class MyResolver(self.etree.Resolver):
+            def resolve(self, url, id, context):
+                assertEqual(url, test_url)
+                return self.resolve_file(
+                    SillyFileLike(
+                        _str('''<!ENTITY myentity "%s">
+                        <!ELEMENT doc ANY>''') % url), context)
+
+        parser.resolvers.add(MyResolver())
+
+        xml = _str('<!DOCTYPE doc SYSTEM "%s"><doc>&myentity;</doc>') % test_url
+        tree = parse(StringIO(xml), parser)
+        root = tree.getroot()
+        self.assertEquals(root.text, test_url)
+
     def test_resolve_filename_dtd(self):
         parse = self.etree.parse
         parser = self.etree.XMLParser(attribute_defaults=True)
@@ -712,6 +809,28 @@ class ETreeOnlyTestCase(HelperTestCase):
         self.assertEquals(
             root[0].attrib, {'default': 'valueB'})
 
+    def test_resolve_file_dtd(self):
+        parse = self.etree.parse
+        parser = self.etree.XMLParser(attribute_defaults=True)
+        assertEqual = self.assertEqual
+        test_url = _str("__nosuch.dtd")
+
+        class MyResolver(self.etree.Resolver):
+            def resolve(self, url, id, context):
+                assertEqual(url, test_url)
+                return self.resolve_file(
+                    open(fileInTestDir('test.dtd'), 'rb'), context)
+
+        parser.resolvers.add(MyResolver())
+
+        xml = _str('<!DOCTYPE a SYSTEM "%s"><a><b/></a>') % test_url
+        tree = parse(StringIO(xml), parser)
+        root = tree.getroot()
+        self.assertEquals(
+            root.attrib,    {'default': 'valueA'})
+        self.assertEquals(
+            root[0].attrib, {'default': 'valueB'})
+
     def test_resolve_empty(self):
         parse = self.etree.parse
         parser = self.etree.XMLParser(load_dtd=True)
@@ -2009,6 +2128,24 @@ class ETreeOnlyTestCase(HelperTestCase):
         self.assertEquals(docinfo.root_name,   'html')
         self.assertEquals(docinfo.doctype, '')
 
+    def test_docinfo_name_only(self):
+        etree = self.etree
+        xml = _bytes('<!DOCTYPE root><root></root>')
+        tree = etree.parse(BytesIO(xml))
+        docinfo = tree.docinfo
+        self.assertEquals(docinfo.encoding,    "UTF-8")
+        self.assertEquals(docinfo.xml_version, "1.0")
+        self.assertEquals(docinfo.public_id,   None)
+        self.assertEquals(docinfo.system_url,  None)
+        self.assertEquals(docinfo.root_name,   'root')
+        self.assertEquals(docinfo.doctype, '<!DOCTYPE root>')
+
+    def test_doctype_name_only_roundtrip(self):
+        etree = self.etree
+        xml = _bytes('<!DOCTYPE root>\n<root/>')
+        tree = etree.parse(BytesIO(xml))
+        self.assertEquals(xml, etree.tostring(tree))
+
     def test_xml_base(self):
         etree = self.etree
         root = etree.XML(_bytes("<root/>"), base_url="http://no/such/url")
@@ -2176,6 +2313,78 @@ class ETreeOnlyTestCase(HelperTestCase):
         result = tostring(a, with_tail=True)
         self.assertEquals(result, _bytes("<a><b/>bTAIL<c/></a>aTAIL"))
 
+    def test_standalone(self):
+        tostring = self.etree.tostring
+        XML = self.etree.XML
+        ElementTree = self.etree.ElementTree
+        Element = self.etree.Element
+
+        tree = Element("root").getroottree()
+        self.assertEquals(None, tree.docinfo.standalone)
+
+        tree = XML(_bytes("<root/>")).getroottree()
+        self.assertEquals(None, tree.docinfo.standalone)
+
+        tree = XML(_bytes(
+            "<?xml version='1.0' encoding='ASCII' standalone='yes'?>\n<root/>"
+            )).getroottree()
+        self.assertEquals(True, tree.docinfo.standalone)
+
+        tree = XML(_bytes(
+            "<?xml version='1.0' encoding='ASCII' standalone='no'?>\n<root/>"
+            )).getroottree()
+        self.assertEquals(False, tree.docinfo.standalone)
+
+    def test_tostring_standalone(self):
+        tostring = self.etree.tostring
+        XML = self.etree.XML
+        ElementTree = self.etree.ElementTree
+
+        root = XML(_bytes("<root/>"))
+
+        tree = ElementTree(root)
+        self.assertEquals(None, tree.docinfo.standalone)
+
+        result = tostring(root, xml_declaration=True, encoding="ASCII")
+        self.assertEquals(result, _bytes(
+            "<?xml version='1.0' encoding='ASCII'?>\n<root/>"))
+
+        result = tostring(root, xml_declaration=True, encoding="ASCII",
+                          standalone=True)
+        self.assertEquals(result, _bytes(
+            "<?xml version='1.0' encoding='ASCII' standalone='yes'?>\n<root/>"))
+
+        tree = ElementTree(XML(result))
+        self.assertEquals(True, tree.docinfo.standalone)
+
+        result = tostring(root, xml_declaration=True, encoding="ASCII",
+                          standalone=False)
+        self.assertEquals(result, _bytes(
+            "<?xml version='1.0' encoding='ASCII' standalone='no'?>\n<root/>"))
+
+        tree = ElementTree(XML(result))
+        self.assertEquals(False, tree.docinfo.standalone)
+
+    def test_tostring_standalone_in_out(self):
+        tostring = self.etree.tostring
+        XML = self.etree.XML
+        ElementTree = self.etree.ElementTree
+
+        root = XML(_bytes(
+            "<?xml version='1.0' encoding='UTF-8' standalone='yes'?>\n<root/>"))
+
+        tree = ElementTree(root)
+        self.assertEquals(True, tree.docinfo.standalone)
+
+        result = tostring(root, xml_declaration=True, encoding="ASCII")
+        self.assertEquals(result, _bytes(
+            "<?xml version='1.0' encoding='ASCII'?>\n<root/>"))
+
+        result = tostring(root, xml_declaration=True, encoding="ASCII",
+                          standalone=True)
+        self.assertEquals(result, _bytes(
+            "<?xml version='1.0' encoding='ASCII' standalone='yes'?>\n<root/>"))
+
     def test_tostring_method_text_encoding(self):
         tostring = self.etree.tostring
         Element = self.etree.Element
@@ -2351,14 +2560,16 @@ class ETreeOnlyTestCase(HelperTestCase):
 
     # helper methods
 
-    def _writeElement(self, element, encoding='us-ascii'):
+    def _writeElement(self, element, encoding='us-ascii', compression=0):
         """Write out element for comparison.
         """
         ElementTree = self.etree.ElementTree
         f = BytesIO()
         tree = ElementTree(element=element)
-        tree.write(f, encoding=encoding)
+        tree.write(f, encoding=encoding, compression=compression)
         data = f.getvalue()
+        if compression:
+            data = zlib.decompress(data)
         return canonicalize(data)
 
 
@@ -2441,12 +2652,187 @@ class ETreeC14NTestCase(HelperTestCase):
         self.assertEquals(_bytes('<a><b></b></a>'),
                           s)
 
+    def test_c14n_gzip(self):
+        tree = self.parse(_bytes('<a>'+'<b/>'*200+'</a>'))
+        f = BytesIO()
+        tree.write_c14n(f, compression=9)
+        s = gzip.GzipFile(fileobj=BytesIO(f.getvalue())).read()
+        self.assertEquals(_bytes('<a>'+'<b></b>'*200+'</a>'),
+                          s)
+
+    def test_c14n_file(self):
+        tree = self.parse(_bytes('<a><b/></a>'))
+        handle, filename = tempfile.mkstemp()
+        try:
+            tree.write_c14n(filename)
+            f = open(filename, 'rb')
+            data = f.read()
+            f.close()
+        finally:
+            os.close(handle)
+            os.remove(filename)
+        self.assertEquals(_bytes('<a><b></b></a>'),
+                          data)
+
+    def test_c14n_file_gzip(self):
+        tree = self.parse(_bytes('<a>'+'<b/>'*200+'</a>'))
+        handle, filename = tempfile.mkstemp()
+        try:
+            tree.write_c14n(filename, compression=9)
+            f = gzip.open(filename, 'rb')
+            data = f.read()
+            f.close()
+        finally:
+            os.close(handle)
+            os.remove(filename)
+        self.assertEquals(_bytes('<a>'+'<b></b>'*200+'</a>'),
+                          data)
+
+    def test_c14n_with_comments(self):
+        tree = self.parse(_bytes('<!--hi--><a><!--ho--><b/></a><!--hu-->'))
+        f = BytesIO()
+        tree.write_c14n(f)
+        s = f.getvalue()
+        self.assertEquals(_bytes('<!--hi-->\n<a><!--ho--><b></b></a>\n<!--hu-->'),
+                          s)
+        f = BytesIO()
+        tree.write_c14n(f, with_comments=True)
+        s = f.getvalue()
+        self.assertEquals(_bytes('<!--hi-->\n<a><!--ho--><b></b></a>\n<!--hu-->'),
+                          s)
+        f = BytesIO()
+        tree.write_c14n(f, with_comments=False)
+        s = f.getvalue()
+        self.assertEquals(_bytes('<a><b></b></a>'),
+                          s)
+
+    def test_c14n_exclusive(self):
+        tree = self.parse(_bytes(
+                '<a xmlns="http://abc" xmlns:y="http://bcd" xmlns:z="http://cde"><z:b/></a>'))
+        f = BytesIO()
+        tree.write_c14n(f)
+        s = f.getvalue()
+        self.assertEquals(_bytes('<a xmlns="http://abc" xmlns:y="http://bcd" xmlns:z="http://cde"><z:b></z:b></a>'),
+                          s)
+        f = BytesIO()
+        tree.write_c14n(f, exclusive=False)
+        s = f.getvalue()
+        self.assertEquals(_bytes('<a xmlns="http://abc" xmlns:y="http://bcd" xmlns:z="http://cde"><z:b></z:b></a>'),
+                          s)
+        f = BytesIO()
+        tree.write_c14n(f, exclusive=True)
+        s = f.getvalue()
+        self.assertEquals(_bytes('<a xmlns="http://abc"><z:b xmlns:z="http://cde"></z:b></a>'),
+                          s)
+
+
+class ETreeWriteTestCase(HelperTestCase):
+    def test_write(self):
+        tree = self.parse(_bytes('<a><b/></a>'))
+        f = BytesIO()
+        tree.write(f)
+        s = f.getvalue()
+        self.assertEquals(_bytes('<a><b/></a>'),
+                          s)
+
+    def test_write_gzip(self):
+        tree = self.parse(_bytes('<a>'+'<b/>'*200+'</a>'))
+        f = BytesIO()
+        tree.write(f, compression=9)
+        s = gzip.GzipFile(fileobj=BytesIO(f.getvalue())).read()
+        self.assertEquals(_bytes('<a>'+'<b/>'*200+'</a>'),
+                          s)
+
+    def test_write_gzip_level(self):
+        tree = self.parse(_bytes('<a>'+'<b/>'*200+'</a>'))
+        f = BytesIO()
+        tree.write(f, compression=0)
+        s0 = f.getvalue()
+
+        f = BytesIO()
+        tree.write(f)
+        self.assertEquals(f.getvalue(), s0)
+
+        f = BytesIO()
+        tree.write(f, compression=1)
+        s = f.getvalue()
+        self.assert_(len(s) <= len(s0))
+        s1 = gzip.GzipFile(fileobj=BytesIO(s)).read()
+
+        f = BytesIO()
+        tree.write(f, compression=9)
+        s = f.getvalue()
+        self.assert_(len(s) <= len(s0))
+        s9 = gzip.GzipFile(fileobj=BytesIO(s)).read()
+
+        self.assertEquals(_bytes('<a>'+'<b/>'*200+'</a>'),
+                          s0)
+        self.assertEquals(_bytes('<a>'+'<b/>'*200+'</a>'),
+                          s1)
+        self.assertEquals(_bytes('<a>'+'<b/>'*200+'</a>'),
+                          s9)
+
+    def test_write_file(self):
+        tree = self.parse(_bytes('<a><b/></a>'))
+        handle, filename = tempfile.mkstemp()
+        try:
+            tree.write(filename)
+            f = open(filename, 'rb')
+            data = f.read()
+            f.close()
+        finally:
+            os.close(handle)
+            os.remove(filename)
+        self.assertEquals(_bytes('<a><b/></a>'),
+                          data)
+
+    def test_write_file_gzip(self):
+        tree = self.parse(_bytes('<a>'+'<b/>'*200+'</a>'))
+        handle, filename = tempfile.mkstemp()
+        try:
+            tree.write(filename, compression=9)
+            f = gzip.open(filename, 'rb')
+            data = f.read()
+            f.close()
+        finally:
+            os.close(handle)
+            os.remove(filename)
+        self.assertEquals(_bytes('<a>'+'<b/>'*200+'</a>'),
+                          data)
+
+    def test_write_file_gzip_parse(self):
+        tree = self.parse(_bytes('<a>'+'<b/>'*200+'</a>'))
+        handle, filename = tempfile.mkstemp()
+        try:
+            tree.write(filename, compression=9)
+            data = etree.tostring(etree.parse(filename))
+        finally:
+            os.close(handle)
+            os.remove(filename)
+        self.assertEquals(_bytes('<a>'+'<b/>'*200+'</a>'),
+                          data)
+
+    def test_write_file_gzipfile_parse(self):
+        tree = self.parse(_bytes('<a>'+'<b/>'*200+'</a>'))
+        handle, filename = tempfile.mkstemp()
+        try:
+            tree.write(filename, compression=9)
+            data = etree.tostring(etree.parse(
+                gzip.GzipFile(filename)))
+        finally:
+            os.close(handle)
+            os.remove(filename)
+        self.assertEquals(_bytes('<a>'+'<b/>'*200+'</a>'),
+                          data)
+
+
 def test_suite():
     suite = unittest.TestSuite()
     suite.addTests([unittest.makeSuite(ETreeOnlyTestCase)])
     suite.addTests([unittest.makeSuite(ETreeXIncludeTestCase)])
     suite.addTests([unittest.makeSuite(ElementIncludeTestCase)])
     suite.addTests([unittest.makeSuite(ETreeC14NTestCase)])
+    suite.addTests([unittest.makeSuite(ETreeWriteTestCase)])
     suite.addTests(
         [make_doctest('../../../doc/tutorial.txt')])
     suite.addTests(
