@@ -4,10 +4,14 @@
 Test cases related to XPath evaluation and the XPath class
 """
 
-import unittest
-from StringIO import StringIO
+import unittest, sys, os.path
 
-from common_imports import etree, HelperTestCase, doctest
+this_dir = os.path.dirname(__file__)
+if this_dir not in sys.path:
+    sys.path.insert(0, this_dir) # needed for Py3
+
+from common_imports import etree, HelperTestCase, _bytes, BytesIO
+from common_imports import doctest, make_doctest
 
 class ETreeXPathTestCase(HelperTestCase):
     """XPath tests etree"""
@@ -75,11 +79,27 @@ class ETreeXPathTestCase(HelperTestCase):
         self.assertEquals([root[0], root[1]],
                           [r.getparent() for r in tree.xpath('/a/b/text()')])
 
+    def test_xpath_list_text_parent_no_smart_strings(self):
+        tree = self.parse('<a><b>FooBar</b><b>BarFoo</b></a>')
+        root = tree.getroot()
+        self.assertEquals(['FooBar', 'BarFoo'],
+                          tree.xpath('/a/b/text()', smart_strings=True))
+        self.assertEquals([root[0], root[1]],
+                          [r.getparent() for r in
+                           tree.xpath('/a/b/text()', smart_strings=True)])
+
+        self.assertEquals(['FooBar', 'BarFoo'],
+                          tree.xpath('/a/b/text()', smart_strings=False))
+        self.assertEquals([False, False],
+                          [hasattr(r, 'getparent') for r in
+                           tree.xpath('/a/b/text()', smart_strings=False)])
+
     def test_xpath_list_unicode_text_parent(self):
-        xml = u'<a><b>FooBar\u0680\u3120</b><b>BarFoo\u0680\u3120</b></a>'
+        xml = _bytes('<a><b>FooBar\\u0680\\u3120</b><b>BarFoo\\u0680\\u3120</b></a>').decode("unicode_escape")
         tree = self.parse(xml.encode('utf-8'))
         root = tree.getroot()
-        self.assertEquals([u'FooBar\u0680\u3120', u'BarFoo\u0680\u3120'],
+        self.assertEquals([_bytes('FooBar\\u0680\\u3120').decode("unicode_escape"),
+                           _bytes('BarFoo\\u0680\\u3120').decode("unicode_escape")],
                           tree.xpath('/a/b/text()'))
         self.assertEquals([root[0], root[1]],
                           [r.getparent() for r in tree.xpath('/a/b/text()')])
@@ -96,10 +116,23 @@ class ETreeXPathTestCase(HelperTestCase):
         self.assertEquals('CqWeRtZuI', results[0])
         self.assertEquals(tree.getroot().tag, results[0].getparent().tag)
 
+    def test_xpath_list_attribute_parent_no_smart_strings(self):
+        tree = self.parse('<a b="BaSdFgHjKl" c="CqWeRtZuI"/>')
+
+        results = tree.xpath('/a/@c', smart_strings=True)
+        self.assertEquals(1, len(results))
+        self.assertEquals('CqWeRtZuI', results[0])
+        self.assertEquals(tree.getroot().tag, results[0].getparent().tag)
+
+        results = tree.xpath('/a/@c', smart_strings=False)
+        self.assertEquals(1, len(results))
+        self.assertEquals('CqWeRtZuI', results[0])
+        self.assertEquals(False, hasattr(results[0], 'getparent'))
+
     def test_xpath_list_comment(self):
         tree = self.parse('<a><!-- Foo --></a>')
         self.assertEquals(['<!-- Foo -->'],
-                          map(repr, tree.xpath('/a/node()')))
+                          list(map(repr, tree.xpath('/a/node()'))))
 
     def test_rel_xpath_boolean(self):
         root = etree.XML('<a><b><c/></b></a>')
@@ -192,7 +225,7 @@ class ETreeXPathTestCase(HelperTestCase):
         root = tree.getroot()
         self.assertEquals(
             [root],
-            e.evaluate('//a'))
+            e('//a'))
 
     def test_xpath_evaluator_tree(self):
         tree = self.parse('<a><b><c></c></b></a>')
@@ -200,11 +233,11 @@ class ETreeXPathTestCase(HelperTestCase):
         e = etree.XPathEvaluator(child_tree)
         self.assertEquals(
             [],
-            e.evaluate('a'))
+            e('a'))
         root = child_tree.getroot()
         self.assertEquals(
             [root[0]],
-            e.evaluate('c'))
+            e('c'))
 
     def test_xpath_evaluator_tree_absolute(self):
         tree = self.parse('<a><b><c></c></b></a>')
@@ -212,14 +245,14 @@ class ETreeXPathTestCase(HelperTestCase):
         e = etree.XPathEvaluator(child_tree)
         self.assertEquals(
             [],
-            e.evaluate('/a'))
+            e('/a'))
         root = child_tree.getroot()
         self.assertEquals(
             [root],
-            e.evaluate('/b'))
+            e('/b'))
         self.assertEquals(
             [],
-            e.evaluate('/c'))
+            e('/c'))
 
     def test_xpath_evaluator_element(self):
         tree = self.parse('<a><b><c></c></b></a>')
@@ -227,7 +260,7 @@ class ETreeXPathTestCase(HelperTestCase):
         e = etree.XPathEvaluator(root[0])
         self.assertEquals(
             [root[0][0]],
-            e.evaluate('c'))
+            e('c'))
         
     def test_xpath_extensions(self):
         def foo(evaluator, a):
@@ -236,7 +269,7 @@ class ETreeXPathTestCase(HelperTestCase):
         tree = self.parse('<a><b></b></a>')
         e = etree.XPathEvaluator(tree, extensions=[extension])
         self.assertEquals(
-            "hello you", e.evaluate("foo('you')"))
+            "hello you", e("foo('you')"))
 
     def test_xpath_extensions_wrong_args(self):
         def foo(evaluator, a, b):
@@ -244,7 +277,7 @@ class ETreeXPathTestCase(HelperTestCase):
         extension = {(None, 'foo'): foo}
         tree = self.parse('<a><b></b></a>')
         e = etree.XPathEvaluator(tree, extensions=[extension])
-        self.assertRaises(TypeError, e.evaluate, "foo('you')")
+        self.assertRaises(TypeError, e, "foo('you')")
 
     def test_xpath_extensions_error(self):
         def foo(evaluator, a):
@@ -252,7 +285,7 @@ class ETreeXPathTestCase(HelperTestCase):
         extension = {(None, 'foo'): foo}
         tree = self.parse('<a/>')
         e = etree.XPathEvaluator(tree, extensions=[extension])
-        self.assertRaises(ZeroDivisionError, e.evaluate, "foo('test')")
+        self.assertRaises(ZeroDivisionError, e, "foo('test')")
 
     def test_xpath_extensions_nodes(self):
         def f(evaluator, arg):
@@ -265,7 +298,7 @@ class ETreeXPathTestCase(HelperTestCase):
 
         x = self.parse('<a/>')
         e = etree.XPathEvaluator(x, extensions=[{(None, 'foo'): f}])
-        r = e.evaluate("foo('World')/result")
+        r = e("foo('World')/result")
         self.assertEquals(2, len(r))
         self.assertEquals('Hoi', r[0].text)
         self.assertEquals('Dag', r[1].text)
@@ -281,7 +314,7 @@ class ETreeXPathTestCase(HelperTestCase):
 
         x = self.parse('<a/>')
         e = etree.XPathEvaluator(x, extensions=[{(None, 'foo'): f}])
-        r = e.evaluate("foo(/*)/result")
+        r = e("foo(/*)/result")
         self.assertEquals(2, len(r))
         self.assertEquals('Hoi', r[0].text)
         self.assertEquals('Dag', r[1].text)
@@ -298,7 +331,7 @@ class ETreeXPathTestCase(HelperTestCase):
 
         x = self.parse('<result>Honk</result>')
         e = etree.XPathEvaluator(x, extensions=[{(None, 'foo'): f}])
-        r = e.evaluate("foo(/*)/result")
+        r = e("foo(/*)/result")
         self.assertEquals(3, len(r))
         self.assertEquals('Hoi',  r[0].text)
         self.assertEquals('Dag',  r[1].text)
@@ -375,14 +408,14 @@ class ETreeXPathTestCase(HelperTestCase):
         e = etree.XPathEvaluator(x)
 
         expr = "/a[@attr=$aval]"
-        r = e.evaluate(expr, aval=1)
+        r = e(expr, aval=1)
         self.assertEquals(0, len(r))
 
-        r = e.evaluate(expr, aval="true")
+        r = e(expr, aval="true")
         self.assertEquals(1, len(r))
         self.assertEquals("true", r[0].get('attr'))
 
-        r = e.evaluate(expr, aval=True)
+        r = e(expr, aval=True)
         self.assertEquals(1, len(r))
         self.assertEquals("true", r[0].get('attr'))
 
@@ -393,7 +426,7 @@ class ETreeXPathTestCase(HelperTestCase):
         element = etree.Element("test-el")
         etree.SubElement(element, "test-sub")
         expr = "$value"
-        r = e.evaluate(expr, value=element)
+        r = e(expr, value=element)
         self.assertEquals(1, len(r))
         self.assertEquals(element.tag, r[0].tag)
         self.assertEquals(element[0].tag, r[0][0].tag)
@@ -424,41 +457,41 @@ class ETreeXPathTestCase(HelperTestCase):
         e = etree.XPathEvaluator(x, extensions=[extension])
         del x
 
-        self.assertRaises(LocalException, e.evaluate, "foo(., 0)")
-        self.assertRaises(LocalException, e.evaluate, "foo(., $value)", value=0)
+        self.assertRaises(LocalException, e, "foo(., 0)")
+        self.assertRaises(LocalException, e, "foo(., $value)", value=0)
 
-        r = e.evaluate("foo(., $value)", value=1)
+        r = e("foo(., $value)", value=1)
         self.assertEqual(len(r), 0)
 
-        r = e.evaluate("foo(.,  1)")
+        r = e("foo(.,  1)")
         self.assertEqual(len(r), 0)
 
-        r = e.evaluate("foo(., $value)", value=2)
+        r = e("foo(., $value)", value=2)
         self.assertEqual(len(r), 0)
 
-        r = e.evaluate("foo(., $value)", value=3)
+        r = e("foo(., $value)", value=3)
         self.assertEqual(len(r), 1)
         self.assertEqual(r[0].tag, "test")
 
-        r = e.evaluate("foo(., $value)", value="false")
+        r = e("foo(., $value)", value="false")
         self.assertEqual(len(r), 1)
         self.assertEqual(r[0].tag, "NODE")
 
-        r = e.evaluate("foo(., 'false')")
+        r = e("foo(., 'false')")
         self.assertEqual(len(r), 1)
         self.assertEqual(r[0].tag, "NODE")
 
-        r = e.evaluate("foo(., 'true')")
+        r = e("foo(., 'true')")
         self.assertEqual(len(r), 1)
         self.assertEqual(r[0].tag, "a")
         self.assertEqual(r[0][0].tag, "test")
 
-        r = e.evaluate("foo(., $value)", value="true")
+        r = e("foo(., $value)", value="true")
         self.assertEqual(len(r), 1)
         self.assertEqual(r[0].tag, "a")
 
-        self.assertRaises(LocalException, e.evaluate, "foo(., 0)")
-        self.assertRaises(LocalException, e.evaluate, "foo(., $value)", value=0)
+        self.assertRaises(LocalException, e, "foo(., 0)")
+        self.assertRaises(LocalException, e, "foo(., $value)", value=0)
 
 
 class ETreeXPathClassTestCase(HelperTestCase):
@@ -467,15 +500,15 @@ class ETreeXPathClassTestCase(HelperTestCase):
         x = self.parse('<a attr="true"/>')
 
         expr = etree.XPath("/a[@attr != 'true']")
-        r = expr.evaluate(x)
+        r = expr(x)
         self.assertEquals(0, len(r))
 
         expr = etree.XPath("/a[@attr = 'true']")
-        r = expr.evaluate(x)
+        r = expr(x)
         self.assertEquals(1, len(r))
 
         expr = etree.XPath( expr.path )
-        r = expr.evaluate(x)
+        r = expr(x)
         self.assertEquals(1, len(r))
 
     def test_xpath_compile_element(self):
@@ -483,22 +516,22 @@ class ETreeXPathClassTestCase(HelperTestCase):
         root = x.getroot()
 
         expr = etree.XPath("./b")
-        r = expr.evaluate(root)
+        r = expr(root)
         self.assertEquals(1, len(r))
         self.assertEquals('b', r[0].tag)
 
         expr = etree.XPath("./*")
-        r = expr.evaluate(root)
+        r = expr(root)
         self.assertEquals(2, len(r))
 
     def test_xpath_compile_vars(self):
         x = self.parse('<a attr="true"/>')
 
         expr = etree.XPath("/a[@attr=$aval]")
-        r = expr.evaluate(x, aval=False)
+        r = expr(x, aval=False)
         self.assertEquals(0, len(r))
 
-        r = expr.evaluate(x, aval=True)
+        r = expr(x, aval=True)
         self.assertEquals(1, len(r))
 
     def test_xpath_compile_error(self):
@@ -513,29 +546,32 @@ class ETreeETXPathClassTestCase(HelperTestCase):
         x = self.parse('<a><b xmlns="nsa"/><b xmlns="nsb"/></a>')
 
         expr = etree.ETXPath("/a/{nsa}b")
-        r = expr.evaluate(x)
+        r = expr(x)
         self.assertEquals(1, len(r))
         self.assertEquals('{nsa}b', r[0].tag)
 
         expr = etree.ETXPath("/a/{nsb}b")
-        r = expr.evaluate(x)
+        r = expr(x)
         self.assertEquals(1, len(r))
         self.assertEquals('{nsb}b', r[0].tag)
 
-    def test_xpath_compile_unicode(self):
-        x = self.parse(u'<a><b xmlns="nsa\uf8d2"/><b xmlns="nsb\uf8d1"/></a>')
+    # disabled this test as non-ASCII characters in namespace URIs are
+    # not acceptable
+    def _test_xpath_compile_unicode(self):
+        x = self.parse(_bytes('<a><b xmlns="http://nsa/\\uf8d2"/><b xmlns="http://nsb/\\uf8d1"/></a>'
+                              ).decode("unicode_escape"))
 
-        expr = etree.ETXPath(u"/a/{nsa\uf8d2}b")
-        r = expr.evaluate(x)
+        expr = etree.ETXPath(_bytes("/a/{http://nsa/\\uf8d2}b").decode("unicode_escape"))
+        r = expr(x)
         self.assertEquals(1, len(r))
-        self.assertEquals(u'{nsa\uf8d2}b', r[0].tag)
+        self.assertEquals(_bytes('{http://nsa/\\uf8d2}b').decode("unicode_escape"), r[0].tag)
 
-        expr = etree.ETXPath(u"/a/{nsb\uf8d1}b")
-        r = expr.evaluate(x)
+        expr = etree.ETXPath(_bytes("/a/{http://nsb/\\uf8d1}b").decode("unicode_escape"))
+        r = expr(x)
         self.assertEquals(1, len(r))
-        self.assertEquals(u'{nsb\uf8d1}b', r[0].tag)
+        self.assertEquals(_bytes('{http://nsb/\\uf8d1}b').decode("unicode_escape"), r[0].tag)
 
-SAMPLE_XML = etree.parse(StringIO("""
+SAMPLE_XML = etree.parse(BytesIO("""
 <body>
   <tag>text</tag>
   <section>
@@ -565,7 +601,7 @@ def setTest2(ctxt, st1):
     return st1[0:2]
 
 def argsTest1(ctxt, s, f, b, st):
-    return ", ".join(map(str, (s, f, b, map(tag, st))))
+    return ", ".join(map(str, (s, f, b, list(map(tag, st)))))
 
 def argsTest2(ctxt, st1, st2):
     st1.extend(st2)
@@ -595,36 +631,43 @@ def xpath():
     
     >>> root = SAMPLE_XML
     >>> e = etree.XPathEvaluator(root, extensions=[extension])
-    >>> e.evaluate("stringTest('you')")
+    >>> e("stringTest('you')")
     'Hello you'
-    >>> e.evaluate(u"stringTest('\xe9lan')")
+    >>> e(_bytes("stringTest('\\\\xe9lan')").decode("unicode_escape"))
     u'Hello \\xe9lan'
-    >>> e.evaluate("stringTest('you','there')")
+    >>> e("stringTest('you','there')")
     Traceback (most recent call last):
     ...
     TypeError: stringTest() takes exactly 2 arguments (3 given)
-    >>> e.evaluate("floatTest(2)")
+    >>> e("floatTest(2)")
     6.0
-    >>> e.evaluate("booleanTest(true())")
+    >>> e("booleanTest(true())")
     False
-    >>> map(tag, e.evaluate("setTest(/body/tag)"))
+    >>> list(map(tag, e("setTest(/body/tag)")))
     ['tag']
-    >>> map(tag, e.evaluate("setTest2(/body/*)"))
+    >>> list(map(tag, e("setTest2(/body/*)")))
     ['tag', 'section']
-    >>> e.evaluate("argsTest1('a',1.5,true(),/body/tag)")
+    >>> e("argsTest1('a',1.5,true(),/body/tag)")
     "a, 1.5, True, ['tag', 'tag', 'tag']"
-    >>> map(tag, e.evaluate("argsTest2(/body/tag, /body/section)"))
+    >>> list(map(tag, e("argsTest2(/body/tag, /body/section)")))
     ['tag', 'section', 'tag', 'tag']
-    >>> e.evaluate("resultTypesTest()")
+    >>> e("resultTypesTest()")
     Traceback (most recent call last):
     ...
     XPathResultError: This is not a node: 'x'
     >>> try:
-    ...     e.evaluate("resultTypesTest2()")
+    ...     e("resultTypesTest2()")
     ... except etree.XPathResultError:
-    ...     print "Got error"
+    ...     print("Got error")
     Got error
     """
+
+if sys.version_info[0] >= 3:
+    xpath.__doc__ = xpath.__doc__.replace(" u'", " '")
+    xpath.__doc__ = xpath.__doc__.replace(" XPathResultError",
+                                          " lxml.etree.XPathResultError")
+    xpath.__doc__ = xpath.__doc__.replace(" exactly 2 arguments",
+                                          " exactly 2 positional arguments")
    
 def test_suite():
     suite = unittest.TestSuite()
@@ -633,8 +676,8 @@ def test_suite():
     suite.addTests([unittest.makeSuite(ETreeETXPathClassTestCase)])
     suite.addTests([doctest.DocTestSuite()])
     suite.addTests(
-        [doctest.DocFileSuite('../../../doc/xpathxslt.txt')])
+        [make_doctest('../../../doc/xpathxslt.txt')])
     return suite
 
 if __name__ == '__main__':
-    print 'to test use test.py %s' % __file__
+    print('to test use test.py %s' % __file__)
